/********************************************************************
| MAIN
|
| Este modulo representa a interface entre o aplicativo e o driver
| Recebe a chamada do aplicativo e chama a api do driver.
|
|********************************************************************/

#include "send.h"
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <io.h>
#include <fcntl.h>
#include <sys\stat.h>

/* Nome do driver */
#define	DEVICE		"FISCNET0"
#define	STX		2


/* Buffers do device driver */
char bufcomm[TAMBUF] = {0x0};    	/* Buffer onde o device driver recebe o comando */
char bufresp[TAMBUF] = {0x1};		/* Buffer de resposta do device driver */
char buftmp[TAMRESP] = {0x0};		/* Buffer temporario de resposta do dd */

int handle;


/*----------------------------------------------------------------------+
| CloseDriver: encerra uma conexao com o device driver FISCNET0.        |
|                                                                       |
|                                                                       |
+----------------------------------------------------------------------*/
void CloseDriver()
{
 if (handle != -1)
	close(handle);
}


/*----------------------------------------------------------------------+
| OpenDriver: abre uma conexao com o device driver FISCNET0.            |
|             retorna um handle; em caso de erro, retorna -1.           |
|                                                                       |
+----------------------------------------------------------------------*/
int OpenDriver()
{
 handle = open(DEVICE, O_BINARY | O_RDWR, S_IREAD | S_IWRITE);
 return handle;
}

/*----------------------------------------------------------------------+
| SendDriver: monta um comando para enviar ao device driver conforme    |
|             API                                                       |
|                                                                       |
+----------------------------------------------------------------------*/
int SendDriver(char cmddriver, char *param, char **pRet)
{
 int tam;

 bufcomm[0] = STX;
 bufcomm[1] = cmddriver;
 bufcomm[2] = 0x0d;
 bufcomm[3] = 0x0a;
 bufcomm[4] = 0x00;

 if (param)
    strcpy (&bufcomm[4], param);

 /* Escreve o comando no arquivo aberto */
 tam = strlen(bufcomm);
 if (write (handle, bufcomm, tam) != tam)
	return -1;

 /* Le o retorno do device driver para o comando enviado */
 tam = read (handle, bufresp, TAMBUF);
 if (tam < 6)
	return -2;

 if (*pRet && (bufresp + 6))
    {
    *pRet = bufresp + 6;

    /* Tira o CR/LF do string */
    tam = strlen(*pRet);

    if (tam > 2)
       *(*pRet + tam - 2) = 0;
    else
       **pRet = 0;
    }

 bufresp[4] = 0x00;
 return atoi (&bufresp[2]);
}


/*----------------------------------------------------------------------+
| main: modulo principal de teste                                       |
|                                                                       |
|                                                                       |
+----------------------------------------------------------------------*/
void main(int argv, char *argc[])
{
 int ret;
 int tam;
 char *pCmd, *pRet;
 char IdCmd[11];
 char ComandoDD[30];
 char ComandoECF[30];

 clrscr();
 if (OpenDriver() == -1)
	{
	printf("Nao foi possivel acessar o device driver %s.\n", DEVICE);
	return;
	}

 /* Verifica os parametros da linha de comando */
 if (argv > 1)
	pCmd = argc[1];
 else
	{
	strcpy (ComandoECF, ";AvancaPapel;Avanco=10");
	pCmd = ComandoECF;
	}

 /* Ajusta o final do comando com \r\n */
 tam = strlen (pCmd);
 strcpy(&pCmd[tam], "\r\n");

 /* Informacoes sobre o device driver */
 printf("Informacoes sobre o Device Driver\n");
 ret = SendDriver('F', "00\r\n", &pRet);
 printf ("Versao      :\t%s\n", (ret == OK)? pRet: "???");

 /* Verifica a porta em uso pelo driver */
 ret = SendDriver('F', "01\r\n", &pRet);
 if (ret == OK)
	printf ("Porta       :\t%s\n", pRet);


 /* Define o identificador do proximo comando */
 randomize();
 ltoa((long)(random(255) + 1), IdCmd, 10);
 strcpy (ComandoDD, "04");
 strcpy (&ComandoDD[2], IdCmd);
 tam = strlen (ComandoDD);
 strcpy (&ComandoDD[tam], "\r\n");
 ret = SendDriver('F', ComandoDD, &pRet);

 /* Proximo IdComando a ser usado em um comando */
 printf ("IdComando   :\t%s\n", pRet);

 ret = SendDriver('F', "026000\r\n", &pRet);
 printf ("Timeout (ms):\t%s\n", pRet);


 /* Envia um comando FiscNET */
 /*ret = SendDriver('@', "LeInteiro;NomeInteiro=\"COO\"\r\n", NULL);*/
 /*ret = SendDriver('@', "LeTexto;NomeTexto=\"ProtocoloSerial\"\r\n", NULL);*/
 /*ret = SendDriver('@', "LeTexto;NomeTexto=\"DadosUltimaReducaoZ\"\r\n", NULL);*/
 /*ret = SendDriver('@', "EmiteLeituraX;\r\n", NULL);*/
 /*ret = SendDriver('@', "LeImpressao;\r\n", NULL);*/
 /*ret = SendDriver('@', "AvancaPapel;Avanco=10\r\n", NULL);*/

 printf ("\nInformacoes sobre o comando:\n");
 ret = SendDriver('@', pCmd, NULL);
 if (ret == PARAM_OK)
	{
	printf ("Comando enviado: %s", pCmd);
	/* Enquanto retornar ocupado, fica em loop aguardando... */
	do
		{
		/* Solicita o retorno do comando */
		ret = SendDriver ('A', NULL, &pRet);

		} while (ret == ERRO_NAO_PRONTO && !kbhit());

	 /* Obtem o codigo do erro e a resposta do ECF */
	 if (ret == OK)
		{
		printf ("Retorno  ECF:\t%s\n", pRet);

		/* Solicita o retorno do comando */
		ret = SendDriver ('E', NULL, &pRet);
		if (ret == OK)
			printf ("Resposta ECF:\t%s\n", pRet);
		else
			printf ("Erro [%s] ao obter resposta do ECF!\n", ret);
		}
	 else
		if (!kbhit())
			printf ("Erro [%d] no retorno do device driver!\n", ret);
		else
			printf ("Processo abortado!\n");
	 }
 else
	printf ("Erro [%d] ao enviar comando!\n", ret);

 printf ("Concluido! Pressione uma tecla para encerrar o programa...\n");
 /* Desinstala os tratadores */
 getch();

 CloseDriver();
}

